import { ObjectModel } from '@awesome-nodes/mvvm/model';
import { ServiceBase as Base } from '@awesome-nodes/mvvm/services';
import { Observable } from 'rxjs';
/**
 * Represents the basic behavior for any service.
 * Accepts only one worker for a specific keyword at a time.
 */
export declare abstract class ServiceBase<T extends ObjectModel | Array<ObjectModel>> extends Base<T> {
    static handleError(e: Error): void;
    /**
     * Creates a new worker instance for the specified worker name.
     *
     * @param workerName
     * @param args
     * @throws ServiceException
     */
    createWorker(workerName: string, ...args: Array<unknown>): Observable<T>;
    /**
     * Implement in the derived class to perform the work and return the work result.
     *
     * @param workerName
     * @param args
     */
    protected abstract doWork(workerName: string, ...args: Array<unknown>): Observable<T>;
}
