import { Nullable } from 'simplytyped';
import { Entity } from './entity';
export declare class EntityCache<T extends Entity> {
    private globalCache;
    private _store;
    get store(): Map<string, T>;
    /**
     * This allows you to specify the source of a cache, which may change from a global source to
     * a cache contained within another entity (for example, a cache of comments in a video).
     *
     * @param value  the new source to be used for the cache. The global cache will be used if
     *                this is null or undefined.
     */
    set store(value: Nullable<Map<string, T>>);
    /**
     * Checks if an entity exists for a given key within the current cache.
     *
     * @param key Key of entity to check for.
     * @returns true if the entity with key is in the cache
     */
    has(key: string): boolean;
    /**
     * Store an entity in the cache
     *
     * @param key key of the entity to store
     * @param entity the entity object to store in the cache
     */
    add(key: string, entity: T): void;
    /**
     * Read a given entity from the cache.
     *
     * @param key key of entity to read from cache
     */
    get(key: string): Nullable<T>;
    /**
     * Returns an entity for the given key.
     *
     * @param key key of entity to store in cache
     * @returns `true` if an element in the Map object existed and has been removed, or `false` if the element does not exist.
     */
    remove(key: string): boolean;
    /**
     * Removes all elements from the cache.
     */
    clear(): void;
}
