import { QueryResult } from '@yukawa/chain-base-angular-domain';
import { Observable } from 'rxjs';
import { Nullable, PlainObject } from 'simplytyped';
import { Entity } from './entity';
import { EntityCache } from './entity-cache';
export declare type EntityKey = number | string | Entity;
export interface IEntityRepositoryService<TEntity extends Entity> {
    /**
     * Convert accepted data to @class Entity object
     *
     * @param json The json data to convert to T
     * @param other
     */
    createInstanceFrom(json: PlainObject, other?: PlainObject): TEntity;
    /**
     * Gets the unique key for an entity of type {@link Entity}.
     * This is used to identify the object within a cache.
     *
     * @param json The json object to get the key from
     * @returns string containing the unique key value
     */
    keyForEntityCache?(json: TEntity): string;
}
export declare type EntityObservable<T extends PlainObject | Entity = PlainObject> = Observable<T | Array<T> | QueryResult<T>>;
export declare class EntityRepository<TEntity extends Entity> extends EntityCache<TEntity> {
    private _repositoryService;
    constructor(_repositoryService: IEntityRepositoryService<TEntity>);
    static isQueryResult<T extends PlainObject>(resultOrQuery: PlainObject | QueryResult<T>): resultOrQuery is QueryResult<T>;
    /** @inheritDoc */
    add(key: string, entity: TEntity): void;
    add<E extends EntityObservable<TEntity>>(observable: EntityObservable, entity?: TEntity): E;
    get(key: EntityKey): Nullable<TEntity>;
    get(observable: Observable<TEntity>, key: string): Observable<TEntity>;
    update(observable: Observable<TEntity>, entity: Entity): Observable<TEntity>;
    remove(key: EntityKey): boolean;
    remove<E extends EntityObservable>(observable: EntityObservable, entity?: TEntity): E;
    /**
     * Instantiates an array of elements as objects from the provided JSON.
     *
     * @returns The array of mapped entity objects
     */
    map(collection: Array<any>, other?: PlainObject): TEntity[];
}
